/*
**	gint:thread:atomic - Thread atomic helper
*/

.section .gint.mapped, "ax"
.align 4

.global	_thread_atomic_start
.type	_thread_atomic_start, @function

/* thread_atomic_start(): Mask interrupts and exceptions */
_thread_atomic_start:
	/* Check if the user is currently into an atomic state and update
	   atomic counter. */
	mov.l	thread_atomic_counter, r1
	mov.l	@r1, r2
	tst	r2, r2
	add	#1, r2
	mov.l	r2, @r1
	bf/s	atomic_start_exit
	mov	#-1, r0

	/* Mask interruptions / exceptions using the IMASK and BL filed of SR */
	stc	sr, r1
	mov	r1, r0
	mov.l	sr_mask, r2
	or	r2, r1
	ldc	r1, sr

	/* Save "old" SR register. */
	mov.l	thread_atomic_sr_save, r1
	mov.l	r0, @r1

atomic_start_exit:
	rts
	nop


.global	_thread_atomic_stop
.type	_thread_atomic_stop, @function

/* thread_atomic_stop(): Unmask (if possible) interrupts / exceptions signals */
_thread_atomic_stop:
	/* Check if the device is currently into an atomic operation, then
	update the counter and, if needed, restore the SR register. */
	mov.l	thread_atomic_counter, r1
	mov.l	@r1, r0
	tst	r0, r0
	bt	atomic_end_error
	cmp/eq	#1, r0
	add	#-1, r0
	mov.l	r0, @r1
	bf/s	atomic_end_exit
	mov	#0, r0

	/* Restore saved SR register data. */
	mov.l	thread_atomic_sr_save, r1
	mov.l	@r1, r0
	ldc	r0, sr
	bra	atomic_end_exit
	nop

atomic_end_error:
	mov	#-1, r0

atomic_end_exit:
	rts
	nop

.align 4
thread_atomic_counter:	.long _thread_atomic_counter
thread_atomic_sr_save:	.long _thread_atomic_sr_save
sr_mask:		.long 0x000000f0



/*
** Global symbols
*/
.global	_thread_atomic_sr_save
.type	_thread_atomic_sr_save, @object
.global	_thread_atomic_counter
.type	_thread_atomic_counter, @object

.align 4
_thread_atomic_sr_save:	.long 0x00000000
_thread_atomic_counter:	.long 0x00000000
